@@ -390,7 +390,16 @@ class DmdLikeCompilerMixin:
     def get_soname_args(self, *args, **kwargs) -> typing.List[str]:
         # LDC and DMD actually do use a linker, but they proxy all of that with
         # their own arguments
-        return Compiler.get_soname_args(self, *args, **kwargs)
+        soargs = []
+        for arg in Compiler.get_soname_args(self, *args, **kwargs):
+            soargs.append('-L=' + arg)
+        return soargs
+
+    def get_allow_undefined_link_args(self) -> typing.List[str]:
+        args = []
+        for arg in self.linker.get_allow_undefined_args():
+            args.append('-L=' + arg)
+        return args
 
 
 class DCompiler(Compiler):
@@ -631,6 +640,9 @@ class GnuDCompiler(DCompiler, GnuCompiler):
 
         return parameter_list
 
+    def get_allow_undefined_link_args(self) -> typing.List[str]:
+        return self.linker.get_allow_undefined_args()
+
 
 class LLVMDCompiler(DmdLikeCompilerMixin, LinkerEnvVarsMixin, BasicLinkerIsCompilerMixin, DCompiler):
     def __init__(self, exelist, version, for_machine: MachineChoice, arch, **kwargs):
@@ -659,6 +671,9 @@ class LLVMDCompiler(DmdLikeCompilerMixin, LinkerEnvVarsMixin, BasicLinkerIsCompi
     def get_pic_args(self):
         return ['-relocation-model=pic']
 
+    def get_std_shared_lib_link_args(self):
+        return ['-shared']
+
     def get_crt_link_args(self, crt_val, buildtype):
         return self.get_crt_args(crt_val, buildtype)
 
